package com.muyu.auth.controller;

import com.muyu.auth.domain.jsr303_group.AutLoginGroup;
import com.muyu.auth.domain.jsr303_group.TelLoginGroup;
import com.muyu.auth.domain.request.UserLoginReq;
import com.muyu.auth.domain.response.TokenLoginRes;
import com.muyu.auth.service.AuthService;
import com.muyu.cloud.system.domain.User;
import com.muyu.common.constant.JwtConstants;
import com.muyu.common.result.Result;
import com.muyu.common.utils.OssUtil;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description: 鉴权控制层
 * @Version 1.0.0
 */
@RestController
@AllArgsConstructor
public class AuthController {

    private final AuthService authService;

    /**
     * 登录
     *
     * @param userLoginReq 登录请求参数
     * @return token
     */
    @PostMapping("/login")
    public Result<TokenLoginRes> login(@Validated(value = AutLoginGroup.class)
                                           @RequestBody UserLoginReq userLoginReq) {
        return Result.success(authService.login(userLoginReq));
    }

    /**
     * loginInfo
     *
     * @param userKey 请求头中获取userKey
     * @return 从redis中获取用户信息
     */
    @GetMapping("/login-info")
    public Result<User> loginInfo(@RequestHeader(JwtConstants.USER_KEY) String userKey) {
        return Result.success(authService.loginInfo(userKey));
    }

    /**
     * loginOut
     *
     * @param userKey 请求头中获取userKey
     * @return 从redis中删除用户信息
     */
    @GetMapping("/login-out")
    public Result<?> loginOut(@RequestHeader(JwtConstants.USER_KEY) String userKey) {
        authService.loginOut(userKey);
        return Result.success();
    }


    /**
     * Oss值
     * @return
     */
    @PostMapping("/getOssKey")
    public Result<?> getOssKey(){
        try {
            return Result.success(OssUtil.getUrl());
        } catch (Exception e) {
            throw new RuntimeException(String.format("【上传文件失败】：返回前端所需参数异常;【原因】：{ %s }",e.getMessage()));
        }
    }

}
