package com.muyu.auth.service.impl;

import com.alibaba.fastjson.JSON;
import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.*;
import com.muyu.auth.domain.request.UserLoginReq;
import com.muyu.auth.domain.response.TokenLoginRes;
import com.muyu.auth.service.AuthService;
import com.muyu.cloud.system.domain.User;
import com.muyu.cloud.system.remote.RemoteUserInfoService;
import com.muyu.common.constant.JwtConstants;
import com.muyu.common.constant.SmsConstants;
import com.muyu.common.constant.TokenConstants;
import com.muyu.common.exception.customException.LoginException;
import com.muyu.common.redis.service.RedisService;
import com.muyu.common.result.Result;
import com.muyu.common.utils.JwtUtils;
import lombok.AllArgsConstructor;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;



import static org.aspectj.weaver.tools.cache.SimpleCacheFactory.path;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description: 鉴权业务实现层
 * @Version 1.0.0
 */
@Service
@AllArgsConstructor
public class AuthServiceImpl implements AuthService {

    private static final Logger log = LogManager.getLogger(AuthServiceImpl.class);
    private final RemoteUserInfoService remoteUserInfoService;

    private final RedisService redisService;
    //加密
    private final BCryptPasswordEncoder bCryptPasswordEncoder;

    private static OSS ossClient;


    private final String bucketName = "wangxin-123";

    private final String path = "https://wangxin-123.oss-cn-shanghai.aliyuncs.com/";

    private static String endPoint = "oss-cn-shanghai.aliyuncs.com";
    private static String accessKeyId = "LTAI5tN9vdDeBioiVbGG3UG3";
    private static String accessKeySecret = "gzhfYLb0UX0zH35DOki9yi1Q35ndgS";
    static {
        ossClient = new OSSClientBuilder().build(
                endPoint,
                accessKeyId,
                accessKeySecret);
        log.info("oss服务连接成功！");
    }

    /**
     * 登录
     *
     * @param userLoginReq 登录请求参数
     * @return token
     */
    @Override
    public TokenLoginRes login(UserLoginReq userLoginReq) {
        Result<User> userResult = remoteUserInfoService.findByUserAut(userLoginReq.getUserAut());
        if (userResult.getCode() != Result.SUCCESS) {
            throw new LoginException(String.format("【登录失败】----【错误信息】---》【 %s 】", userResult.getMsg()));
        }

        User loginUser = userResult.getData();
        if (loginUser == null) {
            throw new LoginException(String.format("【登录失败】----【错误信息】---》【 账号错误 】"));
        }

        if (!bCryptPasswordEncoder.matches(userLoginReq.getUserPwd(), loginUser.getUserPwd())) {
            throw new LoginException(String.format("【登录失败】----【错误信息】---》【 密码错误 】"));
        }

        HashMap<String, Object> map = new HashMap<>();
        String userKey = UUID.randomUUID().toString().replace("-", "");
        map.put(JwtConstants.USER_KEY, userKey);
        map.put(JwtConstants.DETAILS_USER_ID, loginUser.getUserId());
        map.put(JwtConstants.DETAILS_USERNAME, loginUser.getUserName());
        String token = JwtUtils.createToken(map);
        redisService.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey, loginUser, TokenConstants.REFRESH_TIME, TimeUnit.SECONDS);

        return new TokenLoginRes().builder()
                .token(token)
                .expiresTime(TokenConstants.REFRESH_TIME)
                .build();
    }

    /**
     * loginInfo
     *
     * @param userKey 请求头中获取userKey
     * @return 从redis中获取用户信息
     */
    @Override
    public User loginInfo(String userKey) {
        return redisService.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
    }

    /**
     * 登出
     *
     * @param userKey 请求头中获取userKey
     */
    @Override
    public void loginOut(String userKey) {
        redisService.deleteObject(TokenConstants.LOGIN_TOKEN_KEY + userKey);
    }




}
