package com.muyu.common.exception.exceptionHandler;

import com.muyu.common.exception.customException.LoginException;
import com.muyu.common.result.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description: 异常处理器
 * @Version: 1.0.0
 */
@Log4j2
@RestControllerAdvice
public class CustomExceptionHandler {

    @ExceptionHandler(value = LoginException.class)
    public Result<String> runtimeExceptionHandler(Throwable e) {
        StackTraceElement stackTraceElement = e.getStackTrace()[0];
        log.error("程序请求异常:【{}】,该方法报错【类】:{}，【文件】:{}，【报错行数】:{},【方法】:{}",
                e.getMessage(),
                stackTraceElement.getClassName(),
                stackTraceElement.getFileName(),
                stackTraceElement.getLineNumber(),
                stackTraceElement.getMethodName());
        return Result.error(e.getMessage());
    }

    @ExceptionHandler(value = RuntimeException.class)
    public Result<String> runtimeExceptionHandler(RuntimeException e) {
        StackTraceElement stackTraceElement = e.getStackTrace()[0];
        log.error("程序请求异常:【{}】,该方法报错【类】:{}，【文件】:{}，【报错行数】:{},【方法】:{}",
                e.getMessage(),
                stackTraceElement.getClassName(),
                stackTraceElement.getFileName(),
                stackTraceElement.getLineNumber(),
                stackTraceElement.getMethodName());
        return Result.error(e.getMessage());
    }
}
