package com.muyu.search.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.muyu.cloud.search.domain.Car;
import com.muyu.cloud.search.domain.Type;
import com.muyu.cloud.search.domain.Use;
import com.muyu.cloud.search.domain.request.CarListReq;
import com.muyu.cloud.search.domain.response.CarListRes;
import com.muyu.cloud.system.remote.RemoteUserInfoService;
import com.muyu.common.constant.SmsConstants;
import com.muyu.common.redis.service.RedisService;
import com.muyu.common.utils.TelSmsUtils;
import com.muyu.rabbitmq.producer.RabbitMQProducerUtil;
import com.muyu.search.mapper.CarMapper;
import com.muyu.search.service.CarService;
import com.muyu.search.service.UseService;
import com.rabbitmq.client.Channel;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description:
 * @Version 1.0.0
 */
@Service
@AllArgsConstructor
@Log4j2
public class CarServiceImpl implements CarService {

    private final CarMapper carMapper;

    private final RemoteUserInfoService remoteUserInfoService;

    private final RabbitMQProducerUtil rabbitMQProducerUtil;

    private final String QUEUE_NAME = "feeDeduction";

    private final RedisService redisService;

    private final UseService useService;



    @Override
    public CarListRes showList(CarListReq carListReq) {
        PageHelper.startPage(carListReq.getPageNum(), carListReq.getPageSize());
        List<Car> list = carMapper.showList(carListReq);
        PageInfo<Car> pageInfo = new PageInfo<>(list);
        return new CarListRes()
                .builder()
                .cars(pageInfo.getList())
                .total(pageInfo.getTotal())
                .build();
    }

    @Override
    @Transactional
    public void updateStatus(Use use) {
        int i = carMapper.updateStatus(use);
        if (i > 0 && use.getCarStatus() == 0) {
            remoteUserInfoService.feeDeduction(use);
            rabbitMQProducerUtil.basicSendMessage(QUEUE_NAME,use,"结束行程");
        }else if (i > 0 && use.getCarStatus() == 1){
            log.info("添加使用表，数据{}",JSONObject.toJSONString(use));
            useService.addUse(use);
        }
    }

    @Override
    public List<Type> typeAll() {
        return carMapper.typeAll();
    }


    @RabbitListener(queuesToDeclare = @Queue(QUEUE_NAME))
    public void rabbitMQBasicConsumer(Use use , Message message , Channel channel) {
        log.info("当前时间：{} ：RabbitMQConsumerUtil : {}", new Date(), message);
        try {
            // 获取到消息 开始消费
            log.info("消息消费者接收到消息，消息内容：{}", JSONObject.toJSONString(use));


            Long add = redisService.redisTemplate.opsForSet().add(SmsConstants.MSG_KEY + use, message.getMessageProperties().getMessageId());

            if (add != 1) {
                return;
            }
//            TelSmsUtils.sendSms(use.getUserAut(), use.getCarNum());
            log.info("消息发送成功，内容为：{}", JSONObject.toJSONString(use));
            // 消费消息成功之后需要确认
            // long deliveryTag  消息投递序号  自增的数字  在整个队列中唯一 拿到这个序号就相当于拿到这条消息
            // boolean multiple  是否批量确认  true 批量  确认小于等于当前投递序号的消息   false 单个确认
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            log.info("xxx消费者接收到消息，消息内容：{}，消费成功...", message);

        } catch (Exception e) {
            log.error("xxx消费者接收到消息，消息内容：{}，消费消息异常，异常信息：{}", message, e);
            // 消息回退  拒绝消费消息
            // long deliveryTag  消息投递序号  自增的数字  在整个队列中唯一 拿到这个序号就相当于拿到这条消息
            // boolean requeue   是否回到原来的队列
            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(), true);
//                channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            } catch (IOException ex) {
                log.error("xxx消费者接收到消息，消息内容：{}，回退消息异常，异常信息：{}", message, ex);
            }
        }finally {
            try {
                channel.close();
            } catch (Exception e) {
                log.error("xxx消费者关闭Channel异常，消息内容：{}，异常信息：{}", message, e);
            }
        }
    }


}
