package com.muyu.search.service.impl;

import com.muyu.cloud.search.domain.Use;
import com.muyu.common.result.Result;
import com.muyu.common.utils.BigDecimalUtil;
import com.muyu.search.mapper.UseMapper;
import com.muyu.search.service.UseService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description:
 * @Version 1.0.0
 */
@Service
@AllArgsConstructor
public class UseServiceImpl implements UseService {
    private final UseMapper useMapper;

    /**
     * 列表
     * @return 结果集
     */
    @Override
    public List<Use> getListData() {
        List<Use> listData = useMapper.getListData();
        //调用calculateCost方法
        listData.forEach(this::calculateCost);
        return listData;
    }

    @Override
    public int addUse(Use use) {
        return useMapper.addUse(use);
    }

    /**
     * 计算公里数和价格
     */
    private void calculateCost(Use use) {
        //时长
        Long useDuration = use.getUseDuration();
        double num = 0.8;
        double mul = BigDecimalUtil.mul(useDuration, num, 1);
        use.setUseDistance(mul);

        double num1 = 0.1;
        BigDecimal bigDecimal = new BigDecimal(mul)
                .multiply(use.getTypePrice())
                .add(
                        new BigDecimal(num1)
                                .multiply(new BigDecimal(useDuration)
                                )
                ).setScale(2, BigDecimal.ROUND_HALF_UP);
        use.setUseCost(bigDecimal);
    }
}
