package com.muyu.cloud.system.factory;

import com.muyu.cloud.search.domain.Use;
import com.muyu.cloud.system.domain.User;
import com.muyu.cloud.system.remote.RemoteUserInfoService;
import com.muyu.common.constant.ServerNameConstants;
import com.muyu.common.result.Result;
import lombok.extern.log4j.Log4j2;
import org.springframework.cloud.openfeign.FallbackFactory;
import org.springframework.stereotype.Component;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description: 用户远程调用熔断机制
 * @Version: 1.0.0
 */
@Log4j2
public class UserInfoRemoteFactory implements FallbackFactory<RemoteUserInfoService> {
    @Override
    public RemoteUserInfoService create (Throwable cause) {
        /**
         * 通过用户名称查询用户信息
         *
         * @param userAut 用户名称
         *
         * @return 连接用户服务错误报错结果集
         */

        return new RemoteUserInfoService() {
            @Override
            public Result<User> findByUserAut(String userAut) {
                log.error("服务[{}]调用接口发生异常：[{}]", ServerNameConstants.SYSTEM_NAME, cause.getMessage(), cause);
                return Result.error(cause.getMessage());
            }

            @Override
            public Result<?> feeDeduction(Use use) {
                log.error("服务[{}]调用接口发生异常：[{}]", ServerNameConstants.SYSTEM_NAME, cause.getMessage(), cause);
                return Result.error(cause.getMessage());
            }
        };
    }
}
