package com.muyu.cloud.system.remote;

import com.muyu.cloud.search.domain.Use;
import com.muyu.cloud.system.domain.User;
import com.muyu.cloud.system.factory.UserInfoRemoteFactory;
import com.muyu.common.constant.ServerNameConstants;
import com.muyu.common.result.Result;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description: 用户信息远程调用
 * @Version: 1.0.0
 */
@FeignClient(
        name = ServerNameConstants.SYSTEM_NAME,
        // "/user-info" -> controller(requestMapping)
        path = ServerNameConstants.SystemApi.USER_INFO_API,
        contextId = "remoteUserInfoService",
        fallbackFactory = UserInfoRemoteFactory.class
)
public interface RemoteUserInfoService {


    /**
     * 根据账号查询用户信息
     * @param userAut 用户账号
     * @return 用户信息
     */
    @GetMapping("/findByUserAut/{userAut}")
    Result<User> findByUserAut(@PathVariable("userAut") String userAut);

    @PostMapping("/feeDeduction")
    Result<?> feeDeduction(@RequestBody Use use);
}
